function res = lin_elast_stiffness_matrix(x, model, df_info, i, j)
%function res = lin_elast_stiffness_matrix(x, model, df_info, i, j)
%

d_phi_i = evaluate_basis_function_derivative(df_info, x, i);
d_phi_j = evaluate_basis_function_derivative(df_info, x, j);

JIT = df_info.grid.JIT;

% compute strain tensors
strain_i11 = JIT(:, 1, 1) * d_phi_i(1, 1) + JIT(:, 1, 2) * d_phi_i(1, 2);
strain_i22 = JIT(:, 2, 1) * d_phi_i(2, 1) + JIT(:, 2, 2) * d_phi_i(2, 2);
strain_i12 = 0.5*( JIT(:, 1, 1) * d_phi_i(2, 1) + JIT(:, 1, 2) * d_phi_i(2, 2) ...
    + JIT(:, 2, 1) * d_phi_i(1, 1) + JIT(:, 2, 2) * d_phi_i(1, 2) );

strain_j11 = JIT(:, 1, 1) * d_phi_j(1, 1) + JIT(:, 1, 2) * d_phi_j(1, 2);
strain_j22 = JIT(:, 2, 1) * d_phi_j(2, 1) + JIT(:, 2, 2) * d_phi_j(2, 2);
strain_j12 = 0.5*( JIT(:, 1, 1) * d_phi_j(2, 1) + JIT(:, 1, 2) * d_phi_j(2, 2) ...
    + JIT(:, 2, 1) * d_phi_j(1, 1) + JIT(:, 2, 2) * d_phi_j(1, 2) );

% compute divergence
div_i = JIT(:, 1, 1) * d_phi_i(1, 1) + JIT(:, 1, 2) * d_phi_i(1, 2) ...
    + JIT(:, 2, 1) * d_phi_i(2, 1) + JIT(:, 2, 2) * d_phi_i(2, 2);

div_j = JIT(:, 1, 1) * d_phi_j(1, 1) + JIT(:, 1, 2) * d_phi_j(1, 2) ...
    + JIT(:, 2, 1) * d_phi_j(2, 1) + JIT(:, 2, 2) * d_phi_j(2, 2);

% use symmetry
res = eval_affine_decomp_general(...
    @(model)...
        {2*( strain_i11 .* strain_j11 + 2 * strain_i12 .* strain_j12 + strain_i22 .* strain_j22), ...
        div_i .* div_j}, ...
    @(model)[model.lame_mu, model.lame_lambda], ...
    model);

end