function H = house_special(x, idx_operate, idx_nonzero)
% householder transformation which does not put the non-zero entry first
% and allows for "non-block operations"
%
% m does not necessarily equal the m from the main algorithm!

n = numel(x);

if ~isempty(idx_operate)

    i = find(idx_operate == idx_nonzero);

    if isempty(i)
        error('The non-zero index must be contained in the set of indices operated on!');
    end

    idx_perm = 1:n;
    idx_perm(i) = [];
    idx_perm = [i, idx_perm];

    [beta, v] = house(x(idx_perm));
    v(idx_perm) = v;
    
    H = @(A) apply_house_special(A, idx_operate, beta, v);
else
    H = @(A) A;
end

end

function A = apply_house_special(A, idx, beta, v)

A(idx, :) = A(idx, :) - beta * v * (v' * A(idx, :));

end
