function reduced_sim_data = dyn_lin_elast_reduced_simulation(model, reduced_data)
%function sim_data = beam_reduced_simulation(model, reduced_data)
% Performs a detailed simulation of a linear elastic beam
%
% A. Bhatt, 02.05.2017
% Patrick Buchfink, June 2017

% initialization
start_total = tic;

% get parametric coefficients
model.decomp_mode  = 2;
[K_coeff, f_coeff] = model.operators(model, reduced_data);

% evaluate affine decomposition
AN = lincomb_sequence(reduced_data.AN_comp, K_coeff) + reduced_data.AinvMN;

if model.has_time_dependent_rhs_coefficients
    bN   = @(t) model.assemble_time_dependent_h_or_b(model, reduced_data, reduced_data.bN_comp, t);
    dudt = @(t,y) AN * y + bN(t);
else
    bN   = lincomb_sequence(reduced_data.bN_comp, f_coeff);
    dudt = @(t,y) AN * y + bN;
end

jacobian = AN;

% use a numerical method to solve the reduced ODE
switch model.integrator
    case 'rescaled_implicit_midpoint'
        [reduced_sim_data.t,reduced_dofs] = implicit_midpoint(dudt, model.tspan, model.dt, reduced_data.x0, jacobian);
    case 'rescaled_implicit_euler'
        [reduced_sim_data.t,reduced_dofs] = implicit_euler(dudt, model.tspan, model.dt, reduced_data.x0, jacobian);
    otherwise
        error('Unknown numerical integrator: %s', model.integrator);
end
reduced_dofs = reduced_dofs';
reduced_sim_data.reduced_dofs = reduced_dofs;

% compute Hamiltonian
if isfield(model, 'is_symplectic') && model.is_symplectic
    reduced_sim_data = reduced_data.eval_reduced_Hamiltonian(model, reduced_data, K_coeff, f_coeff, reduced_sim_data);
end
    
% return results:
reduced_sim_data.total_time = toc(start_total);
end